/*
 * JStandard.java - The Main file for JStandard
 * Copyright (C) 2000, Vinod Kiran
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jstandard;

import java.io.*;
import jstandard.parser.*;
import java.util.Vector;
import java.util.Enumeration;
import javax.swing.JOptionPane;

import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.msg.*;

public class JStandard
{
	View parent;
    public static String NAME = "JStandard";
	protected String filename;
	protected JStandardEngine engine;
	
	public JStandard(View view)
    {
    	parent = view;
    }

    public void checkStandards( String s )
    {
    	errorSource.clear();
		filename = s;
       	//JStandardProgressDialog pd = new JStandardProgressDialog(parent);
//        JStandardRules rules = new JStandardRules();
		engine = new JStandardEngine();
        try
        {
//            pd.setProgressText("Reading Source File........");
			JavaParser.parse(s,engine);
//            pd.setProgressText("Analysing Results.........");
			engine.analyseWarnings();
//            pd.setProgressText("Preparing List of Warnings.........");
            showResults();
		}
        catch(FileNotFoundException e)
        {
			JOptionPane.showMessageDialog(parent,"OOPS! Dont know the location for "+filename);
		}
        catch(ParseException pe)
        {
        	String[] message = new String[]{"Error! While parsing ("+filename+")",
							"Line:"+pe.getErrorLine()+" Col:"+pe.getErrorColumn()};
			JOptionPane.showMessageDialog(parent,message);
		}
        finally
        {
//            pd.hide();
//        	pd.dispose();
        }
    }

    private void showResults()
    {
		Enumeration enum = engine.getWarnings();
		if ( enum == null)
        {
        	GUIUtilities.message(parent,"jstandard.noviolations",null);
			return;
		}

		JStandardWarning warning ;
        while ( enum.hasMoreElements() )
        {
			warning = (JStandardWarning) enum.nextElement();
            errorSource.addError(ErrorSource.WARNING,
            						filename,
            						warning.getLine()-1,
									0,
									warning.getColumn(),
                                    warning.getMsg());
        }
    }

	public static void start() {
		errorSource = new DefaultErrorSource(NAME);
		EditBus.addToNamedList(ErrorSource.ERROR_SOURCES_LIST,errorSource);
		EditBus.addToBus(errorSource);
	}

	public static void stop() {
		EditBus.removeFromNamedList(ErrorSource.ERROR_SOURCES_LIST,errorSource);
		EditBus.removeFromBus(errorSource);
	}

	private static DefaultErrorSource errorSource;
}
