/*
 * JStandardEngine.java - The heart of the system.
 * Copyright (C) 2000, Vinod Kiran
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jstandard;

import java.util.Vector;
import java.util.Enumeration;
import jstandard.parser.Token;
import org.gjt.sp.jedit.jEdit;

public class JStandardEngine implements JStandardConstants
{
	protected Vector messages;
	protected JStandardRules rules;
	public JStandardEngine()
	{
		messages = new Vector();
	}
	
	public void addMessage(short description, String name, int line, int col)
	{
		String desc = jEdit.getProperty("jstandard.warning."+description,"Unknown Warning");
		messages.addElement(new JStandardWarning(description, desc, name, line, col));
	}
	
	public void addMessage(short description, Token msgToken)
	{
		String desc = jEdit.getProperty("jstandard.warning."+description,"Unknown Warning");
		messages.addElement(new JStandardWarning(description,desc,
												 msgToken.image,
												 msgToken.beginLine,
												 msgToken.beginColumn));
	}
	
	public void analyseWarnings()
	{
		rules = new JStandardRules();
		Vector filteredMessages = new Vector();
		Enumeration enum = messages.elements();
		JStandardWarning warning = null;
		while ( enum.hasMoreElements())
		{
			warning = (JStandardWarning)enum.nextElement();
			if ( rules.isRuleEnabled(warning.getType()) == false )
				continue;
			
			switch ( warning.getType()) {
				case NAMING_CLASS :
					if ( !rules.verifyClassName(warning.getToken()) )
					{
						filteredMessages.addElement(warning);
					}
					break;
				case NAMING_INTERFACE:
					if ( !rules.verifyInterfaceName(warning.getToken()) )
					{
						filteredMessages.addElement(warning);
					}
					break;
				case NAMING_METHOD:
					if ( !rules.verifyMethodName(warning.getToken()) )
					{
						filteredMessages.addElement(warning);
					}
					break;
				case NAMING_ATTRIBUTE:
					if ( !rules.verifyFieldName(warning.getToken()) )
					{
						filteredMessages.addElement(warning);
					}
					break;
				case NAMING_LOCALVAR:
					if ( !rules.verifyLocalVariableName(warning.getToken()) )
					{
						filteredMessages.addElement(warning);
					}
					break;
				case NAMING_FINAL_VAR:
					if ( !rules.verifyFinalFieldName(warning.getToken()) )
					{
						filteredMessages.addElement(warning);
					}
					break;
				case NAMING_FINAL_ATTR:
					if ( !rules.verifyFinalFieldName(warning.getToken()) )
					{
						filteredMessages.addElement(warning);
					}
					break;
				case NAMING_PARAMETER:
					if ( !rules.verifyParameterName(warning.getToken()) )
					{
						filteredMessages.addElement(warning);
					}
					break;
				case NAMING_LABEL:
					if ( !rules.verifyLabelName(warning.getToken()) )
					{
						filteredMessages.addElement(warning);
					}
					break;
				default:
					filteredMessages.addElement(warning);
			}
		}
		messages = null;
		messages = filteredMessages;
		filteredMessages = null;
	}
	
	public Enumeration getWarnings()
	{
		if ( messages.size() == 0 )
			return null;
		return messages.elements();
	}
}
