/*
 * JStandardOptionPane.java - Options for JStandard
 * Copyright (C) 2000, Vinod Kiran
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jstandard;

import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.gui.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;


/**

 This is the option pane that jEdit displays for Plugin Options.

 */
public class JStandardOptionPane extends AbstractOptionPane
{
	JTabbedPane tabs   = new JTabbedPane();
    Box naming         = new Box(BoxLayout.Y_AXIS);
    Box coding  	   = new Box(BoxLayout.Y_AXIS);
    Box flow 	 	   = new Box(BoxLayout.Y_AXIS);
    Box clarity  	   = new Box(BoxLayout.Y_AXIS);
	
	// naming
	private JCheckBox verifyClassName = new JCheckBox("Verify Class Names");
	private JCheckBox verifyMethodName = new JCheckBox("Verify Method Names");
	private JCheckBox verifyFieldName = new JCheckBox("Verify Field Names");
	private JCheckBox verifyParameters = new JCheckBox("Verify Method Parameter Names");
	private JCheckBox verifyFinalFields = new JCheckBox("Verify Final Fields");
	private JCheckBox verifyFinalVars = new JCheckBox("Verify Final Variables");
	private JCheckBox verifyLocalVariables = new JCheckBox("Verify Local Variables");
	private JCheckBox verifyInterfaceName = new JCheckBox("Verify Interface Names");
	private JCheckBox verifyLabelName = new JCheckBox("Verify Label Names");
	
	//clarity
	private JCheckBox verifyMultipleTypes = new JCheckBox("Check for Multiple Class Declarations in a File");
	private JCheckBox verifyMultipleVars = new JCheckBox("Check for Multiple Variables declared in one line");
	private JCheckBox verifyMultipleAttrs = new JCheckBox("Check for Multiple Attributes declared in one line");
	private JCheckBox verifyUsageTerenary = new JCheckBox("Check for Usage of ?: operator");
	private JCheckBox verifyDotStarImport = new JCheckBox("Check for .* imports");
	
	// flow
	private JCheckBox verifySwitch = new JCheckBox("Check for Switch without DEFAULT");
	private JCheckBox verifyEmptyCatch = new JCheckBox("Check for an Empty Catch Block");
	private JCheckBox verifyEmptyFinally = new JCheckBox("Check for an Empty Finally Block");
	private JCheckBox verifyEmptyIf = new JCheckBox("Check for IF Statement without a Block");
	private JCheckBox verifyEmptyFor = new JCheckBox("Check for FOR Statement without a Block");
	private JCheckBox verifyEmptyWhile = new JCheckBox("Check for WHILE Statement without a Block");
	private JCheckBox verifyEmptyStatement = new JCheckBox("Check for an Empty Statement");

	// coding
	private JCheckBox verifyPackage = new JCheckBox("Check for presence of package Statement");
	private JCheckBox verifyDuplicateImport = new JCheckBox("Check for Duplicate imports");
	private JCheckBox verifyJavaLangImport = new JCheckBox("Check for unnecessary import - java.lang.*");
	private JCheckBox verifyImportCurPackage = new JCheckBox("Check for import of current package");
	private JCheckBox verifyAbstractInterface = new JCheckBox("Check for Interfaces declared ABSTRACT");
	private JCheckBox verifyAbstractInterfaceMethod = new JCheckBox("Check for Interface Methods declared ABSTRACT");
	private JCheckBox verifyConstructorVisibilty = new JCheckBox("Check for Constructor with default visibility");

	public JStandardOptionPane()
	{
		super("JStandard");
		setLayout(gridBag = new GridBagLayout());
	}

	private void buildUI()
	{
		//************************* naming options
		verifyClassName.setSelected( jEdit.getProperty("jstandard.naming.classname","T").equals("T") );
		verifyClassName.setToolTipText("Class Names should begin with a Capital Letter");
		
		verifyMethodName.setSelected(jEdit.getProperty("jstandard.naming.methodname","T").equals("T"));
		verifyMethodName.setToolTipText("Method Names should begin with a LowerCase Letter");
		
		verifyFieldName.setSelected(jEdit.getProperty("jstandard.naming.fieldname","T").equals("T"));
		verifyFieldName.setToolTipText("Field Names should begin with a LowerCase Letter");
		
		verifyParameters.setSelected(jEdit.getProperty("jstandard.naming.parameters","T").equals("T"));
		verifyParameters.setToolTipText("Parameters should begin with a LowerCase Letter");
		
		verifyFinalFields.setSelected(jEdit.getProperty("jstandard.naming.finalfields","T").equals("T"));
		verifyFinalFields.setToolTipText("Final (Constant) fields should be in UPPERCASE");
		
		verifyFinalVars.setSelected(jEdit.getProperty("jstandard.naming.finalvars","T").equals("T"));
		verifyFinalVars.setToolTipText("Final (Constant) Variables should be in UPPERCASE");

		verifyLocalVariables.setSelected(jEdit.getProperty("jstandard.naming.localvariable","T").equals("T"));
		verifyLocalVariables.setToolTipText("Local Variables should begin with a LowerCase letter");
		
		verifyInterfaceName.setSelected(jEdit.getProperty("jstandard.naming.interface","T").equals("T"));
		verifyInterfaceName.setToolTipText("Interface Names should begin with a Capital Letter");
		
		verifyLabelName.setSelected(jEdit.getProperty("jstandard.naming.labelname","T").equals("T"));
		verifyLabelName.setToolTipText("Label Names should begin with a LowerCase letter");
		
		//clarity
		verifyMultipleTypes.setSelected(jEdit.getProperty("jstandard.clarity.types","T").equals("T"));
		verifyMultipleTypes.setToolTipText("Each Source File Should have only one Public Class");
		
		verifyMultipleVars.setSelected(jEdit.getProperty("jstandard.clarity.vars","T").equals("T"));
		verifyMultipleVars.setToolTipText("Each Variable must be declared in a Seperate Line");
		
		verifyMultipleAttrs.setSelected(jEdit.getProperty("jstandard.clarity.attrs","T").equals("T"));
		verifyMultipleAttrs.setToolTipText("Each Attribute must be declared in a Seperate Line");

		verifyUsageTerenary.setSelected(jEdit.getProperty("jstandard.clarity.terenary","T").equals("T"));
		verifyUsageTerenary.setToolTipText("Usage of ?: is often confusing. Use if...else");

		verifyDotStarImport.setSelected(jEdit.getProperty("jstandard.clarity.dotstar","T").equals("T"));
		verifyDotStarImport.setToolTipText("Import by Specific Class Name");

		// flow
		verifyEmptyCatch.setSelected(jEdit.getProperty("jstandard.flow.catch","T").equals("T"));
		verifyEmptyCatch.setToolTipText("Catch Blocks should not be Empty");
		
		verifyEmptyFinally.setSelected(jEdit.getProperty("jstandard.flow.finally","T").equals("T"));
		verifyEmptyFinally.setToolTipText("Finally Blocks should not be Empty");
		
		verifySwitch.setSelected(jEdit.getProperty("jstandard.flow.switch","T").equals("T"));
		verifySwitch.setToolTipText("Every Switch statement should have a DEFAULT option");
		
		verifyEmptyIf.setSelected(jEdit.getProperty("jstandard.flow.if","T").equals("T"));
		verifyEmptyIf.setToolTipText("Body of IF Statements must be enclosed in {}");

		verifyEmptyFor.setSelected(jEdit.getProperty("jstandard.flow.for","T").equals("T"));
		verifyEmptyFor.setToolTipText("Body of FOR Statements must be enclosed in {}");
		
		verifyEmptyWhile.setSelected(jEdit.getProperty("jstandard.flow.while","T").equals("T"));
		verifyEmptyWhile.setToolTipText("Body of WHILE Statements must be enclosed in {}");

		verifyEmptyStatement.setSelected(jEdit.getProperty("jstandard.flow.empty","T").equals("T"));
		verifyEmptyStatement.setToolTipText("Empty Statment.");

		// coding
		verifyPackage.setSelected(jEdit.getProperty("jstandard.coding.package","T").equals("T"));
		verifyPackage.setToolTipText("Every Class must belong to a Package");
		
		verifyDuplicateImport.setSelected(jEdit.getProperty("jstandard.coding.duplicateimport","T").equals("T"));
		verifyDuplicateImport.setToolTipText("Duplicate Import Statement!");
		
		verifyJavaLangImport.setSelected(jEdit.getProperty("jstandard.coding.javalang","T").equals("T"));
		verifyJavaLangImport.setToolTipText("java.lang.* is imported by default");
		
		verifyImportCurPackage.setSelected(jEdit.getProperty("jstandard.coding.curpackage","T").equals("T"));
//		verifyImportCurPackage.setToolTipText("You dont have to import the package of this class");

		verifyAbstractInterface.setSelected(jEdit.getProperty("jstandard.coding.abstractinterface","T").equals("T"));
		verifyAbstractInterface.setToolTipText("Interfaces should not be declared ASBTRACT");
		
		verifyAbstractInterfaceMethod.setSelected(jEdit.getProperty("jstandard.coding.interfacemethod","T").equals("T"));
		verifyAbstractInterfaceMethod.setToolTipText("Interface Methods should not be declared ASBTRACT");

		verifyConstructorVisibilty.setSelected(jEdit.getProperty("jstandard.coding.constructor","T").equals("T"));
		verifyConstructorVisibilty.setToolTipText("Constructors must be public/private/protected");

		// naming
		naming.add(verifyClassName);
		naming.add(verifyMethodName);
		naming.add(verifyFieldName);
		naming.add(verifyParameters);
		naming.add(verifyFinalFields);
		naming.add(verifyFinalVars);
		naming.add(verifyLocalVariables);
		naming.add(verifyInterfaceName);
		naming.add(verifyLabelName);
		tabs.addTab("Naming",naming);

		//************************* Coding Options
		coding.add(verifyPackage);
		coding.add(verifyDuplicateImport);
		coding.add(verifyJavaLangImport);
		coding.add(verifyImportCurPackage);
		coding.add(verifyAbstractInterface);
		coding.add(verifyAbstractInterfaceMethod);
		coding.add(verifyConstructorVisibilty);
		tabs.addTab("Coding Style",coding);
		//flow
		flow.add(verifySwitch);
		flow.add(verifyEmptyCatch);
		flow.add(verifyEmptyFinally);
		flow.add(verifyEmptyIf);
		flow.add(verifyEmptyFor);
		flow.add(verifyEmptyWhile);
		flow.add(verifyEmptyStatement);
		tabs.addTab("Program Flow",flow);
		// clarity
		clarity.add(verifyMultipleTypes);
		clarity.add(verifyMultipleVars);
		clarity.add(verifyMultipleAttrs);
		clarity.add(verifyUsageTerenary);
		clarity.add(verifyDotStarImport);
		tabs.addTab("Code Clarity",clarity);

		addComponent(tabs);
	}
	/**
	 * Called when the options dialog's "ok" button is clicked.
	 * This should save any properties being edited in this option
	 * pane.
	 */
	public void save()
	{
		jEdit.setProperty("jstandard.naming.classname", verifyClassName.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.naming.methodname", verifyMethodName.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.naming.fieldname", verifyFieldName.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.naming.parameters", verifyParameters.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.naming.finalfields", verifyFinalFields.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.naming.finalvars", verifyFinalVars.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.naming.interface", verifyInterfaceName.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.naming.localvariable", verifyLocalVariables.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.naming.labelname", verifyLabelName.isSelected() ? "T" : "F");

		jEdit.setProperty("jstandard.flow.catch", verifyEmptyCatch.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.flow.finally", verifyEmptyFinally.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.flow.switch", verifySwitch.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.flow.if", verifyEmptyIf.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.flow.while", verifyEmptyWhile.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.flow.for", verifyEmptyFor.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.flow.empty", verifyEmptyStatement.isSelected() ? "T" : "F");

		jEdit.setProperty("jstandard.coding.package", verifyPackage.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.coding.duplicateimport", verifyDuplicateImport.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.coding.javalang", verifyJavaLangImport.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.coding.curpackage", verifyImportCurPackage.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.coding.abstractinterface", verifyAbstractInterface.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.coding.interfacemethod", verifyAbstractInterfaceMethod.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.coding.constructor", verifyConstructorVisibilty.isSelected() ? "T" : "F");

		jEdit.setProperty("jstandard.clarity.types", verifyMultipleTypes.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.clarity.attrs", verifyMultipleAttrs.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.clarity.vars", verifyMultipleVars.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.clarity.terenary", verifyUsageTerenary.isSelected() ? "T" : "F");
		jEdit.setProperty("jstandard.clarity.dotstar", verifyDotStarImport.isSelected() ? "T" : "F");
	}
	
	
    public static void main(String[] args)
    {
		JFrame fr = new JFrame("Test Frame");
		fr.getContentPane().add(new JStandardOptionPane());
		fr.pack();
		fr.setVisible(true);
    }
	/**
	 * This method should create the option pane's GUI.
	 */
	protected void _init()
	{
		buildUI();
	}
	
	/**
	 * Adds a component to the option pane.
	 * @param comp The component
	 */
	protected void addComponent(Component comp)
	{
		GridBagConstraints cons = new GridBagConstraints();
		cons.gridy = y++;
		cons.gridheight = 1;
		cons.gridwidth = cons.REMAINDER;
		cons.fill = GridBagConstraints.HORIZONTAL;
		cons.anchor = GridBagConstraints.WEST;
		cons.weightx = 1.0f;
		gridBag.setConstraints(comp,cons);
		add(comp);
	}
}
