/*
 * JStandardRules.java - The rules Engine for JStandard
 * Copyright (C) 2000, Vinod Kiran
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jstandard;
import gnu.regexp.*;
import org.gjt.sp.jedit.jEdit;
import java.util.Vector;
import java.util.Hashtable;
import jstandard.parser.*;

public class JStandardRules implements JStandardConstants
{
    public final String REG_EXP_CLASS = "^[A-Z]";
    public final String REG_EXP_INTERFACE = "^[A-Z]";
    public final String REG_EXP_METHOD = "^[a-z]";
    public final String REG_EXP_PARAMETER = "^[a-z]";
    public final String REG_EXP_LOCALVARIABLE = "^[a-z]";
    public final String REG_EXP_FIELD = "^[a-z]";
    public final String REG_EXP_FINAL_FIELD = "^[A-Z_]+$";
    public final String REG_EXP_LABEL = "^[a-z]";

	private Hashtable selected_rules ;
	public JStandardRules()
    {
		selected_rules = new Hashtable();
		
		selected_rules.put(new Integer(NAMING_CLASS),jEdit.getProperty("jstandard.naming.classname", "T"));
		selected_rules.put(new Integer(NAMING_METHOD),jEdit.getProperty("jstandard.naming.methodname", "T"));
		selected_rules.put(new Integer(NAMING_ATTRIBUTE),jEdit.getProperty("jstandard.naming.fieldname", "T"));
		selected_rules.put(new Integer(NAMING_PARAMETER),jEdit.getProperty("jstandard.naming.parameters", "T"));
		selected_rules.put(new Integer(NAMING_FINAL_ATTR),jEdit.getProperty("jstandard.naming.finalfields", "T"));
		selected_rules.put(new Integer(NAMING_FINAL_VAR),jEdit.getProperty("jstandard.naming.finalvars", "T"));
		selected_rules.put(new Integer(NAMING_INTERFACE),jEdit.getProperty("jstandard.naming.interface", "T"));
		selected_rules.put(new Integer(NAMING_LOCALVAR),jEdit.getProperty("jstandard.naming.localvariable", "T"));
		selected_rules.put(new Integer(NAMING_LABEL),jEdit.getProperty("jstandard.naming.labelname", "T"));

		selected_rules.put(new Integer(FLOW_EMPTY_CATCH),jEdit.getProperty("jstandard.flow.catch", "T"));
		selected_rules.put(new Integer(FLOW_EMPTY_FINALLY),jEdit.getProperty("jstandard.flow.finally", "T"));
		selected_rules.put(new Integer(FLOW_NO_DEFAULT),jEdit.getProperty("jstandard.flow.switch", "T"));
		selected_rules.put(new Integer(FLOW_IF_NOBLOCK),jEdit.getProperty("jstandard.flow.if", "T"));
		selected_rules.put(new Integer(FLOW_WHILE_NOBLOCK),jEdit.getProperty("jstandard.flow.while","T"));
		selected_rules.put(new Integer(FLOW_FOR_NOBLOCK),jEdit.getProperty("jstandard.flow.for", "T"));
		selected_rules.put(new Integer(FLOW_NOBODY),jEdit.getProperty("jstandard.flow.empty", "T"));

		selected_rules.put(new Integer(CODING_NO_PACKAGE),jEdit.getProperty("jstandard.coding.package", "T"));
		selected_rules.put(new Integer(CODING_DUPLICATE_IMPORT),jEdit.getProperty("jstandard.coding.duplicateimport", "T"));
		selected_rules.put(new Integer(CODING_IMPORT_JAVALANG),jEdit.getProperty("jstandard.coding.javalang", "T"));
		selected_rules.put(new Integer(CODING_IMPORT_CURPACKAGE),jEdit.getProperty("jstandard.coding.curpackage", "T"));
		selected_rules.put(new Integer(CODING_ABSTRACT_INTERFACE),jEdit.getProperty("jstandard.coding.abstractinterface", "T"));
		selected_rules.put(new Integer(CODING_ABSTRACT_INTERFACEMETHOD),jEdit.getProperty("jstandard.coding.interfacemethod", "T"));
		selected_rules.put(new Integer(CODING_CONSTRUCTOR_VISIBILITY),jEdit.getProperty("jstandard.coding.constructor", "T"));

		selected_rules.put(new Integer(CLARITY_MULTIPLE_CLASS),jEdit.getProperty("jstandard.clarity.types", "T"));
		selected_rules.put(new Integer(CLARITY_MULTIPLE_ATTR_DECLARATIONS),jEdit.getProperty("jstandard.clarity.attrs", "T"));
		selected_rules.put(new Integer(CLARITY_MULTIPLE_VAR_DECLARATIONS),jEdit.getProperty("jstandard.clarity.vars", "T"));
		selected_rules.put(new Integer(CLARITY_USAGE_TERENRY_OPERATOR),jEdit.getProperty("jstandard.clarity.terenary", "T"));
		selected_rules.put(new Integer(CLARITY_DOTSTAR_IMPORT),jEdit.getProperty("jstandard.clarity.dotstar",  "T"));
    }

    public boolean verifyLabelName(String s )
    {
        return checkValidity(REG_EXP_LABEL, s);
    }

    public boolean verifyMethodName(String s )
    {
        return checkValidity(REG_EXP_METHOD, s);
    }

    public boolean verifyClassName(String s )
    {
        return checkValidity(REG_EXP_CLASS, s);
    }

    public boolean verifyInterfaceName(String s )
    {
        return checkValidity(REG_EXP_INTERFACE, s);
    }

    public boolean verifyParameterName(String s )
    {
        return checkValidity(REG_EXP_PARAMETER, s);
    }

    public boolean verifyFinalFieldName(String s )
    {
        return checkValidity(REG_EXP_FINAL_FIELD, s);
    }

    public boolean verifyLocalVariableName(String s )
    {
        return checkValidity(REG_EXP_LOCALVARIABLE, s);
    }

    public boolean verifyFieldName(String s )
    {
        return checkValidity(REG_EXP_FIELD, s);
    }

    private boolean checkValidity(String pattern, String text)
    {
    	try {
	        RE regexp = new RE(pattern,0);
    	    return !(regexp.getMatch(text)==null);
        }
        catch(Exception ree)
        {
            return false;
        }
    }
	
	public boolean isRuleEnabled(short ruleIndex)
	{
		String ans = (String)selected_rules.get(new Integer(ruleIndex));
		if ( ans == null )
			return false;
		return ans.equals("T");
	}
}
